import boto3
import json
import pandas as pd
from jsonpath_ng import jsonpath
from jsonpath_ng.ext import parse

#####################################

def fields(args, data):
    f = args.get('fields',[])
    if not f: return data
    d = {}
    for je in f: 
        for i in parse(je).find(data):
            d.update({je:i.value})
    return d

#####################################

def found(args, data):
    f = args.get('filter',[])
    if not f: return True
    for je in f:
        if [match.value for match in parse(je).find([data])]:
            return True
    return False

#####################################

class paginate():
    def __init__(self, session, **kargs):
        self.json = []
        pargs = {k:v for k,v in kargs.items() if k not in ['ep', 'service', 'fields', 'filter'] if kargs}
        for page in session.client(kargs.get('service')).get_paginator(kargs.get('ep')).paginate(**pargs):
            self.json = [fields(kargs, i.value) for i in self.jsonpath_expression().find(page) if found(kargs, i.value)]
        self.df = pd.json_normalize(self.json)

            
    def __str__(self):
        return json.dumps(self.json)

#####################################

class ec2_paginate(paginate):
    def __init__(self, session, **kargs):
        kargs.update({'ep':self.name()})
        kargs.update({'service':'ec2'})
        paginate.__init__(self, session, **kargs)

#####################################

class org_paginate(paginate):
    def __init__(self, session, **kargs):
        kargs.update({'ep':self.name()})
        kargs.update({'service':'organizations'})
        paginate.__init__(self, session, **kargs)

#####################################

class s3_paginate(paginate):
    def __init__(self, session, **kargs):
        kargs.update({'ep':self.name()})
        kargs.update({'service':'s3'})
        paginate.__init__(self, session, **kargs)

#####################################

class cw_paginate(paginate):
    def __init__(self, session, **kargs):
        kargs.update({'ep':self.name()})
        kargs.update({'service':'cloudwatch'})
        paginate.__init__(self, session, **kargs)

#####################################

class ec_paginate(paginate):
    def __init__(self, session, **kargs):
        kargs.update({'ep':self.name()})
        kargs.update({'service':'elasticache'})
        paginate.__init__(self, session, **kargs)

#####################################



import awswrangler as wr
from functools import partial

#####################

def query(session, s, **kargs):
    generated_query = generate_query(s, **kargs) 
    return partial(wr.athena.read_sql_query,
                   database=kargs.get('db'),
                   boto3_session=session)(generated_query)

#####################

def generate_query(s, **kargs):
    for key,val in kargs.items():
        s = s.replace('{' + key + '}', str(val)) if f'{key}' in s else s
    return s

#####################




from jsonpath_ng import parse
from .paginate import cw_paginate

from datetime import datetime as dt
from datetime import timedelta as td

#####################################

class get_metric_data(cw_paginate):
    def name(self): return __class__.__name__        
    def jsonpath_expression(self): return parse('MetricDataResults[*]')

#####################################

def _get_metric_stat(s, ccid, name, period, days, stat):

  ss = kargs.get('session')
  p = kargs.get('Period')
  mn = kargs.get('MetricName')
  s = kargs.get('Stat')
  ns = f"AWS/{kargs.get('Namespace')}"
  d = kargs.get('Dimensions')
  et = dt.now()
  st = et - td(days=kargs.get('days'))

  try:
      md = ss.client('cloudwatch').get_metric_statistics(StartTime=st, EndTime=et, 
                                                        Namespace=ns,
                                                        MetricName=mv,
                                                        Dimensions=[d],
                                                        Period=p, 
                                                        Statistics=[s])
      return md['Datapoints'][0][stat] if len(md['Datapoints']) > 0 else None
  except Exception as e:
        print(e)
        return None

#####################################
# Utility
#####################################

def _get_metric_data(**kargs):

  ss = kargs.get('session')
  p = kargs.get('Period')
  mn = kargs.get('MetricName')
  s = kargs.get('Stat')
  ns = f"AWS/{kargs.get('Namespace')}"
  d = kargs.get('Dimensions')
  et = dt.now()
  st = et - td(days=kargs.get('days'))
  f = kargs.get('field')

  try:
      md = get_metric_data(ss, 
                              fields=[f], 
                              StartTime=st, 
                              EndTime=et, 
                              MetricDataQueries=[
                                  {"Id": "m1", 
                                   "MetricStat": {
                                       "Metric": {
                                           "Namespace": ns,
                                           "MetricName": mn,
                                           "Dimensions": [d]
                                       },
                                       "Period": p, 
                                       "Stat": s
                                   }
                                  }
                              ]).json;
      return [m[f] if f in m and m[f] else None for m in md][0]
  except Exception as e:
        print(e)
        return None